# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lacss.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0blacss.proto\x12\x0ftrackmate.lacss\"\xba\x01\n\x05Image\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\r\n\x05width\x18\x02 \x01(\x04\x12\x0f\n\x07\x63hannel\x18\x03 \x01(\x04\x12%\n\x05\x64type\x18\x04 \x01(\x0e\x32\x16.trackmate.lacss.DType\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12\r\n\x05\x64\x65pth\x18\x06 \x01(\x04\x12\x13\n\x0bvoxel_dim_x\x18\x07 \x01(\x02\x12\x13\n\x0bvoxel_dim_y\x18\x08 \x01(\x02\x12\x13\n\x0bvoxel_dim_z\x18\t \x01(\x02\"\xcd\x01\n\x08Settings\x12\x15\n\rmin_cell_area\x18\x01 \x01(\x02\x12\x1b\n\x13remove_out_of_bound\x18\x02 \x01(\x08\x12\x0f\n\x07scaling\x18\x03 \x01(\x02\x12\x0f\n\x07nms_iou\x18\x04 \x01(\x02\x12\x1b\n\x13\x64\x65tection_threshold\x18\x05 \x01(\x02\x12\x1e\n\x16segmentation_threshold\x18\x06 \x01(\x02\x12\x16\n\x0ereturn_polygon\x18\x07 \x01(\x08\x12\x16\n\x0e\x63\x65ll_size_hint\x18\x08 \x01(\x02\"[\n\x05Input\x12+\n\x08settings\x18\x01 \x01(\x0b\x32\x19.trackmate.lacss.Settings\x12%\n\x05image\x18\x02 \x01(\x0b\x32\x16.trackmate.lacss.Image\"(\n\x05Point\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"@\n\x07Polygon\x12\r\n\x05score\x18\x01 \x01(\x02\x12&\n\x06points\x18\x02 \x03(\x0b\x32\x16.trackmate.lacss.Point\"\x84\x01\n\x04Mesh\x12\r\n\x05score\x18\x01 \x01(\x02\x12%\n\x05verts\x18\x02 \x03(\x0b\x32\x16.trackmate.lacss.Point\x12\r\n\x05\x66\x61\x63\x65s\x18\x03 \x03(\x04\x12\'\n\x07normals\x18\x04 \x03(\x0b\x32\x16.trackmate.lacss.Point\x12\x0e\n\x06values\x18\x05 \x03(\x02\"`\n\x03Roi\x12+\n\x07polygon\x18\x01 \x01(\x0b\x32\x18.trackmate.lacss.PolygonH\x00\x12%\n\x04mesh\x18\x02 \x01(\x0b\x32\x15.trackmate.lacss.MeshH\x00\x42\x05\n\x03roi\"-\n\x07Results\x12\"\n\x04rois\x18\x01 \x03(\x0b\x32\x14.trackmate.lacss.Roi*\x14\n\x05\x44Type\x12\x0b\n\x07\x46LOAT32\x10\x00\x32K\n\x05Lacss\x12\x42\n\x0cRunDetection\x12\x16.trackmate.lacss.Input\x1a\x18.trackmate.lacss.Results\"\x00\x42\x30\n$fiji.plugin.trackmate.detector.lacssB\x08LacssMsgb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lacss_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n$fiji.plugin.trackmate.detector.lacssB\010LacssMsg'
  _globals['_DTYPE']._serialized_start=910
  _globals['_DTYPE']._serialized_end=930
  _globals['_IMAGE']._serialized_start=33
  _globals['_IMAGE']._serialized_end=219
  _globals['_SETTINGS']._serialized_start=222
  _globals['_SETTINGS']._serialized_end=427
  _globals['_INPUT']._serialized_start=429
  _globals['_INPUT']._serialized_end=520
  _globals['_POINT']._serialized_start=522
  _globals['_POINT']._serialized_end=562
  _globals['_POLYGON']._serialized_start=564
  _globals['_POLYGON']._serialized_end=628
  _globals['_MESH']._serialized_start=631
  _globals['_MESH']._serialized_end=763
  _globals['_ROI']._serialized_start=765
  _globals['_ROI']._serialized_end=861
  _globals['_RESULTS']._serialized_start=863
  _globals['_RESULTS']._serialized_end=908
  _globals['_LACSS']._serialized_start=932
  _globals['_LACSS']._serialized_end=1007
# @@protoc_insertion_point(module_scope)
